#!/usr/bin/perl 
use Term::ANSIColor;
use strict;
use List::Util qw(shuffle);
#    ____          _                           _                 
#   / ___|   __ _ | |_  _ __  ___   _ __ ___  (_) ____ ___  _ __ 
#   \___ \  / _` || __|| '__|/ _ \ | '_ ` _ \ | ||_  // _ \| '__|
#    ___) || (_| || |_ | |  | (_) || | | | | || | / /|  __/| |   
#   |____/  \__,_| \__||_|   \___/ |_| |_| |_||_|/___|\___||_|  
#   Color Edition!
#   http://jonsatrom.com  
#	jon@satrom.com
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version. http://www.gnu.org/licenses/gpl.html
#   Satromizer.pl was written by jon.satrom and bensyverson.
#
#
#   Use the Satromizer to glitch, corrupt, and destroy directories of data!
#
#   It works by scrambling chunks of bytes within your files.
#   Give it a directory name (source), a destination directory,
#   a suggested "header" byte size to avoid,
#   the number of operations to perform,
#   a minimum chunk size and a maximum chunk size,
#   a minimum distance and a maximum distance


# ======== command line options =========
my $dirname 		= $ARGV[0];
my $destdir 		= $ARGV[1];
my $headersize 		= $ARGV[2];
my $numoperations 	= $ARGV[3];
my $minchunk 		= $ARGV[4];
my $maxchunk 		= $ARGV[5];
my $mindist 		= $ARGV[6];
my $maxdist 		= $ARGV[7];

($minchunk, $maxchunk) = ($maxchunk, $minchunk) if ($minchunk > $maxchunk);
($mindist, $maxdist) = ($maxdist, $mindist) if ($mindist > $maxdist);

print
  "   \n"
. "   ", colored("+ ===================================================================+ ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
. "   ", colored(" | ", 'white on_blue'), colored("     ____          _                           _                 ", 'magenta on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
. "   ", colored(" | ", 'white on_blue'), colored("   / ___|   __ _ | |_  _ __  ___   _ __ ___  (_) ____ ___  _ __  ", 'magenta on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
. "   ", colored(" | ", 'white on_blue'), colored("   \\___ \\  / _` || __|| '__|/ _ \\ | '_ ` _ \\ | ||_  // _ \\| '__| ", 'magenta on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
. "   ", colored(" | ", 'white on_blue'), colored("    ___) || (_| || |_ | |  | (_) || | | | | || | / /|  __/| |    ", 'magenta on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
. "   ", colored(" | ", 'white on_blue'), colored("   |____/  \\__,_| \\__||_|   \\___/ |_| |_| |_||_|/___|\\___||_|    ", 'magenta on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
. "   ", colored(" | ", 'white on_blue'), colored("                                                                 ", 'magenta on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n";

print 		
	 	  "   ", colored(" +===================================================================+ ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
	 	. "   ", colored(" | ", 'white on_blue'), colored("                                                                 ", 'blue on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"  
		.  "   ", colored(" | ", 'white on_blue'), colored("        dirname: ", 'blue on_cyan'), colored("$dirname",'black on_cyan', 'blink'), colored("                           ", 'blue on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" | ", 'white on_blue'), colored("    destination: ", 'blue on_cyan'), colored("$destdir",'black on_cyan', 'blink'), colored("                           ", 'blue on_cyan'), colored(" | ",'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" | ", 'white on_blue'), colored("     headersize: ", 'blue on_cyan'), colored("$headersize",'black on_cyan', 'blink'), colored("                                             ", 'blue on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" | ", 'white on_blue'), colored("     num of ops: ", 'blue on_cyan'), colored("$numoperations",'black on_cyan', 'blink'), colored("                                         .....", 'blue on_cyan'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" | ", 'white on_blue'), colored("      min chunk: ", 'blue on_cyan'), colored("$minchunk",'black on_cyan', 'blink'), colored("                                       ", 'blue on_cyan'), colored("      ", 'blue on_green'), colored(" ", 'blue on_magenta'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" | ", 'white on_blue'), colored("      max chunk: ", 'blue on_cyan'), colored("$maxchunk",'black on_cyan', 'blink'), colored("                            ", 'blue on_cyan'), colored("              ", 'blue on_red'), colored(" ", 'blue on_green'), colored(" ", 'blue on_cyan'), colored("   ", 'blue on_red'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" | ", 'white on_blue'), colored("       min dist: ", 'blue on_cyan'), colored("$mindist",'black on_cyan', 'blink'), colored("                                  ", 'blue on_cyan'), colored(" ", 'blue on_yellow'), colored("...,        ", 'blue on_magenta', 'blink'), colored(" ", 'blue on_green'), colored(" |", 'white on_magenta', 'blink'), colored(" ", 'white on_red', 'blink'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" | ", 'white on_blue'), colored("       max dist: ", 'blue on_cyan'), colored("$maxdist",'black on_cyan', 'blink'), colored("                                           ", 'blue on_cyan'), colored(" ", 'blue on_red'), colored(" ", 'blue on_green'), colored("+  ", 'white on_magenta'), colored(" | ", 'white on_cyan'), colored("\\", 'blue on_black'), "\n"
		. "   ", colored(" | ", 'white on_blue'), colored("                                               _ = = ", 'white on_cyan'), colored("   ", 'blue on_green'), colored("  XX", 'magenta on_yellow'), colored("  ", 'blue on_red'), colored("X", 'blue on_black', 'blink'), colored(" ", 'blue on_cyan'), colored(" ", 'blue on_magenta'), colored(" | ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored(" +===================================================================+ ", 'white on_blue'), colored("\\", 'blue on_black'), "\n"
		.  "   ", colored("\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\", 'blue on_black'), "\n";


#=============================
# Figure out the smallest file
#=============================

my $smallest = 0;
my $imagecount = 0;
opendir(DIR, $dirname) or die "    CANT'T OPEN DIRECTORY: $dirname: $!";
while (defined(my $file = readdir(DIR))) {										# iterate over all the files in $dirname
	unless ($file eq '.' or $file eq '..' or $file =~ /^\..*$/) {				# skip the "." and ".." entries, and any file starting with "."	
		my $size = -s "$dirname/$file";
		$smallest = $size if ($smallest == 0 || $smallest > $size); 			# set $smallest to the size of the current file if if this is the first or smallest file we've seen
		$imagecount++;
	}
}
print "    ", colored(" SMALLEST FILESIZE: $smallest    \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\", 'white on_blue'), "\n";

my $lastpos = $smallest - $maxchunk;
print "     LAST AVAILABLE CHUNK POSITION (SMALLEST FILE - MAXCHUNK): $lastpos \n";
print colored("    ///////////////////////////////////////////////////////////////////////", 'blue on_black'), "\n";



die "SORRY; HEADER SIZE IS LARGER THAN THE LAST AVAILABLE CHUNK POSITION;  " .
		"TRY A SMALLER HEADER OR SMALLER CHUNK. \n" if ($headersize > $lastpos) ;

#==============================
# Build up a list of operations
#==============================

my @operations;

for(my $i=0; $i < $numoperations; $i++) {
	# determine chunk position
	my $chunkposition;
	my $chunksize;
	my $movdist;
	while (1) {
		$chunkposition 	= int( rand( $lastpos - $headersize + 1) ) + $headersize;		# pick a chunk position that's between the end of the header and the last available spot
		$chunksize 		= int( rand( $maxchunk - $minchunk + 1) ) + $minchunk;			# pick a random chunk size between our range
		$movdist 		= int( rand( $maxdist  - $mindist + 1) ) + $mindist;			# pick a random move distance between our range
		
		last unless (($chunkposition + $movdist) < $headersize or ($chunkposition + $chunksize + $movdist) > $smallest);	# make sure we don't go over our range
	}
	
	$operations[$i] = {															# populate our array entry with this hash
		'chunkposition' => $chunkposition, 
		'chunksize'		=> $chunksize,
		'movdist'		=> $movdist,
	};
	
	if ($numoperations < 255) {
		my @array = qw(white blue white yellow cyan red black);	
		my @errey = qw(on_red on_cyan on_magenta on_green on_green on_white on_black on_blue);
		my @randchar = qw(! # $ % ^ & * _ - = + : " ? > < X);
		my $randchar = $randchar[rand @randchar];
		my $ralamant = $array[rand @array];
		my $relement = $errey[rand @errey];
		
		
print color("$ralamant","on_black"), "$randchar"; my $randchar = $randchar[rand @randchar]; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black"), "$randchar"; my $randchar = $randchar[rand @randchar]; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black"), "$randchar"; my $randchar = $randchar[rand @randchar]; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black"), "  $randchar "; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","$relement"),"OPERATION"; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","$relement")," $i: "; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","$relement")," GOTO: "; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black")," $operations[$i]{'chunkposition'} "; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","$relement"),"SELECT:"; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black")," $operations[$i]{'chunksize'} BYTES "; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","$relement")," & MOVE IT "; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black")," $operations[$i]{'movdist'} "; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","$relement")," W/IN THE FILE. "; my $randchar = $randchar[rand @randchar]; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black"), " $randchar"; my $randchar = $randchar[rand @randchar]; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black"), " $randchar"; my $randchar = $randchar[rand @randchar]; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black"), " $randchar"; my $randchar = $randchar[rand @randchar]; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; print color("$ralamant","on_black"), " $randchar"; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; my $randchar = $randchar[rand @randchar]; print color("$ralamant","on_black"), " $randchar"; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; my $randchar = $randchar[rand @randchar]; print color("$ralamant","on_black"), "$randchar"; my $ralamant = $array[rand @array]; my $relement = $errey[rand @errey]; my $randchar = $randchar[rand @randchar]; print color("$ralamant","on_black"), "       $randchar \\ \n"; color("reset");
	}
}

print "    ", colored("/////////////////////////////////////////////////////////////////// *FINISHED", 'white on_blue'), "\n";


#=====================================
# Perform the operations on every file
#=====================================

my $currentimage = 0;
my $cursorpos = 0;
my $direction = 1;
print "    ", colored("\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\", 'white on_blue'), colored(" PROCESSING $imagecount IMAGES...\n", 'white on_blue', 'blink');
opendir(DIR, $dirname) or die "CAN'T OPEN DIRECTORY: $dirname: $!";
while (defined(my $file = readdir(DIR))) {										# iterate over all the files in $dirname
	unless ($file eq '.' or $file eq '..' or $file =~ /^\..*$/) {				# skip the "." and ".." entries, and any file starting with "."
		# animate the cursor to provide a little feedback as we process images...
		
		my @array = qw(white blue white yellow cyan red black);	
		my @errey = qw(on_red on_cyan on_magenta on_green on_green on_white on_black on_blue);
		my @randchar = qw(! # $ % ^ & * _ - = + : " ? > < X);
		my $randchar = $randchar[rand @randchar];
		my $ralamant = $array[rand @array];
		my $relement = $errey[rand @errey];
		
		my $percent = int($currentimage / $imagecount * 100);
		print "    $file (" . ($percent < 10 ? ' ' : '') . $percent . '%) ' . ' ' x $cursorpos;
		if ($direction == 1) {
			print color("$ralamant","$relement")," \\\\\\ " . "\n";
			$direction = -1 if ($cursorpos > 30);
		} else {
			print color("$ralamant","$relement")," /// " . "\n";
			$direction = 1 if ($cursorpos < 1);
		}
		$cursorpos += $direction;
		$currentimage++;
		
		# end cursor animation
		
		local( *FILE);															# slurp in the file, part 1
		open(FILE, "< :raw", "$dirname/$file" ) or die "WHOOPS $!\n";			# open the file
		my $data = do { local( $/ ) ; <FILE> } ;								# slurp in the file, part 2
		close FILE;																# close the file
		if ($data) {															# if we have data
			foreach(@operations) {																	# then for each operation
				my $chunk 		= substr($data, $$_{'chunkposition'}, $$_{'chunksize'});			# select our chunk
				substr($data, $$_{'chunkposition'}, $$_{'chunksize'}) = '';							# delete the chunk
				$$_{'movdist'} -= $$_{'chunksize'} if ($$_{'movdist'} > 0);							# if we're moving the chunk down, compensate for the fact that we just deleted the chunk
				my $firsthalf 	= substr($data, 0, $$_{'chunkposition'} + $$_{'movdist'});			# grab everything from the beginning up to where we're moving our chunk
				my $lasthalf 	= substr($data, $$_{'chunkposition'} + $$_{'movdist'});				# grab everything from where we're moving our chunk on
				$data = $firsthalf . $chunk . $lasthalf if ($chunk and $firsthalf and $lasthalf);	# stick it all together in the right order
			}
			open (OUTPUT, "> :raw", "$destdir/$file") or die "could not open $destdir/$file: $! \n";
			print OUTPUT $data;
			close OUTPUT;
		} else {
			print " # error";
		}
	}
}

		my @array = qw(white blue white yellow cyan red black);	
		my @errey = qw(on_red on_cyan on_magenta on_green on_green on_white on_black on_blue);
		my @randchar = qw(! # $ % ^ & * _ - = + : " ? > < X);
		my $randchar = $randchar[rand @randchar];
		my $ralamant = $array[rand @array];
		my $relement = $errey[rand @errey];
		my $randchar = $randchar[rand @randchar];
		my $ralamant = $array[rand @array];
		my $relement = $errey[rand @errey];
		print   color("$ralamant","on_black"), " $randchar";
		my $ralamant = $array[rand @array];
		my $relement = $errey[rand @errey];		
		my $randchar = $randchar[rand @randchar];
		print   color("$ralamant","on_black"), "       $randchar \\ \n";
				 color("reset");

print "    ", colored("/////////////////////////////////////////////////////////////////// * FINISHED SATROMIZING!   ", 'white on_blue'), "\n";


